/**
 * inversify v.2.0.0-beta.3 - A lightweight IoC container written in TypeScript.
 * Copyright (c) 2015 Remo H. Jansen
 * MIT inversify.io/LICENSE
 * http://inversify.io
 */
function _proxyGetter(kernel, serviceIdentifier, proto, key, resolve) {
    var globalInjectionMap = kernel._injectedProperties;
    var getter = function () {
        var instanceInjections = globalInjectionMap.get(this);
        if (instanceInjections == null) {
            instanceInjections = new Map();
            globalInjectionMap.set(this, instanceInjections);
        }
        if (!instanceInjections.has(key)) {
            var val = resolve(kernel, serviceIdentifier);
            instanceInjections.set(key, val);
        }
        return instanceInjections.get(key);
    };
    var setter = function (newVal) {
        var instanceInjections = globalInjectionMap.get(this);
        if (instanceInjections == null) {
            instanceInjections = new Map();
            globalInjectionMap.set(this, instanceInjections);
        }
        instanceInjections.set(key, newVal);
    };
    delete proto[key];
    Object.defineProperty(proto, key, {
        configurable: true,
        enumerable: true,
        get: getter,
        set: setter
    });
}
function makePropertyInjectDecorator(kernel) {
    return function (serviceIdentifier) {
        return function (proto, key) {
            var resolve = function (krln, srvId) {
                return krln.get(srvId);
            };
            _proxyGetter(kernel, serviceIdentifier, proto, key, resolve);
        };
    };
}
function makePropertyInjectNamedDecorator(kernel) {
    return function (serviceIdentifier, named) {
        return function (proto, key) {
            var resolve = function (krln, srvId) {
                return krln.getNamed(srvId, named);
            };
            _proxyGetter(kernel, serviceIdentifier, proto, key, resolve);
        };
    };
}
function makePropertyInjectTaggedDecorator(kernel) {
    return function (serviceIdentifier, key, value) {
        return function (proto, propertyName) {
            var resolve = function (krln, srvId) {
                return krln.getTagged(srvId, key, value);
            };
            _proxyGetter(kernel, serviceIdentifier, proto, propertyName, resolve);
        };
    };
}
function makePropertyMultiInjectDecorator(kernel) {
    return function (serviceIdentifier) {
        return function (proto, key) {
            var resolve = function (krln, srvId) {
                return krln.getAll(srvId);
            };
            _proxyGetter(kernel, serviceIdentifier, proto, key, resolve);
        };
    };
}
export { makePropertyInjectDecorator, makePropertyMultiInjectDecorator, makePropertyInjectTaggedDecorator, makePropertyInjectNamedDecorator };
