/**
 * inversify v.2.0.0-beta.2 - A lightweight IoC container written in TypeScript.
 * Copyright (c) 2015 Remo H. Jansen
 * MIT inversify.io/LICENSE
 * http://inversify.io
 */
"use strict";
function getProxy(kernel, serviceIdentifier) {
    var _val = null;
    var getter = function () {
        if (_val === null) {
            _val = kernel.get(serviceIdentifier);
        }
        return _val;
    };
    var setter = function (newVal) {
        _val = newVal;
    };
    return {
        getter: getter,
        setter: setter,
    };
}
function makePropertyInjectDecorator(kernel) {
    return function (serviceIdentifier) {
        return function (target, targetKey) {
            var proxy = getProxy(kernel, serviceIdentifier);
            if (delete target[targetKey]) {
                Object.defineProperty(target, targetKey, {
                    configurable: true,
                    enumerable: true,
                    get: proxy.getter,
                    set: proxy.setter
                });
            }
        };
    };
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = makePropertyInjectDecorator;
